@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2007 by PCMan (Hong Jen Yee)   *
+ *   Copyright (C) 2007, 2008 by PCMan (Hong Jen Yee)   *
  *   pcman.tw@gmail.com   *
  *                                                                         *
  *   mw program is free software; you can redistribute it and/or modify  *
@@ -55,12 +55,13 @@ static void main_win_finalize( GObject* obj );
 static void create_nav_bar( MainWin* mw, GtkWidget* box);
 GtkWidget* add_nav_btn( MainWin* mw, const char* icon, const char* tip, GCallback cb, gboolean toggle);
 // GtkWidget* add_menu_item(  GtkMenuShell* menu, const char* label, const char* icon, GCallback cb, gboolean toggle=FALSE );
-static void rotate_image( MainWin* mw, GdkPixbufRotation angle );
+static void rotate_image( MainWin* mw, int angle );
 static void show_popup_menu( MainWin* mw, GdkEventButton* evt );
 
 /* signal handlers */
 static gboolean on_delete_event( GtkWidget* widget, GdkEventAny* evt );
 static void on_size_allocate( GtkWidget* widget, GtkAllocation    *allocation );
+static gboolean on_win_state_event( GtkWidget* widget, GdkEventWindowState* state );
 static void on_zoom_fit( GtkToggleButton* btn, MainWin* mw );
 static void on_zoom_fit_menu( GtkMenuItem* item, MainWin* mw );
 static void on_full_screen( GtkWidget* btn, MainWin* mw );
@@ -88,6 +89,13 @@ static void on_drag_data_received( GtkWidget* widget, GdkDragContext *drag_conte
 static void on_delete( GtkWidget* btn, MainWin* mw );
 static void on_about( GtkWidget* menu, MainWin* mw );
 
+static GdkPixbuf* RotateByEXIF(const char* FileName, GdkPixbuf* pix);
+static void updateTitle(const char *filename, MainWin *mw );
+
+void on_flip_vertical( GtkWidget* btn, MainWin* mw );
+void on_flip_horizontal( GtkWidget* btn, MainWin* mw );
+static int trans_angle_to_id(int i);
+static int get_new_angle( int orig_angle, int rotate_angle );
 
 // Begin of GObject-related stuff
 
@@ -107,6 +115,7 @@ void main_win_class_init( MainWinClass* klass )
     widget_class->delete_event = on_delete_event;
     widget_class->size_allocate = on_size_allocate;
     widget_class->key_press_event = on_key_press_event;
+    widget_class->window_state_event = on_win_state_event;
 }
 
 void main_win_finalize( GObject* obj )
@@ -186,7 +195,7 @@ void main_win_init( MainWin*mw )
 
     // build toolbar
     mw->tooltips = gtk_tooltips_new();
-#if GTK_CHECK_VERSION(2, 10, 0)
+#if GTK_CHECK_VERSION(2, 8, 0)
     g_object_ref_sink(mw->tooltips);
 #else
     gtk_object_sink((GtkObject*)mw->tooltips);
@@ -268,18 +277,115 @@ gboolean on_delete_event( GtkWidget* widget, GdkEventAny* evt )
     return TRUE;
 }
 
+static GdkPixbuf* RotateByEXIF(const char* FileName, GdkPixbuf* pix)
+{
+    GdkPixbuf* tmppixbuf = pix;
+#if GTK_CHECK_VERSION( 2, 12, 0 )
+    // apply orientation provided by EXIF (Use gtk+ 2.12 specific API)
+    tmppixbuf = gdk_pixbuf_apply_embedded_orientation(pix);
+    g_object_unref( pix );
+#else
+    // use jhead functions
+    ResetJpgfile();
+
+    // Start with an empty image information structure.
+    memset(&ImageInfo, 0, sizeof(ImageInfo));
+
+    if (!ReadJpegFile( FileName, READ_METADATA)) return;
+
+    // Do Rotate
+    switch(ImageInfo.Orientation)
+    {
+        case 0:	// Undefined
+        case 1:	// Normal
+          break;
+        case 2:	// flip horizontal: left right reversed mirror
+          tmppixbuf = gdk_pixbuf_flip(pix, TRUE);
+          g_object_unref( pix );
+          break;
+        case 3:	// rotate 180
+          tmppixbuf = gdk_pixbuf_rotate_simple(pix, 180);
+          g_object_unref( pix );
+          break;
+        case 4:	// flip vertical: upside down mirror
+          tmppixbuf = gdk_pixbuf_flip(pix, FALSE);
+          g_object_unref( pix );
+          break;
+        case 5:	// transpose: Flipped about top-left <--> bottom-right axis.
+          tmppixbuf = gtk_pixbuf_flip(pix, FALSE);
+          g_object_unref( pix );
+          pix = tmppixbuf;
+          tmppixbuf = gtk_pixbuf_rotate_simple(pix, 270);
+          g_object_unref( pix );
+          break;
+        case 6:	// rotate 90: rotate 90 cw to right it.
+          tmppixbuf = gdk_pixbuf_rotate_simple(pix, 270);
+          g_object_unref( pix );
+          break;
+        case 7:	// transverse: flipped about top-right <--> bottom-left axis
+          tmppixbuf = gtk_pixbuf_flip(pix, FALSE);
+          g_object_unref( pix );
+          pix = tmppixbuf;
+          tmppixbuf = gtk_pixbuf_rotate_simple(pix, 90);
+          g_object_unref( pix );
+          break;
+        case 8:	// rotate 270: rotate 270 to right it.
+          tmppixbuf = gdk_pixbuf_rotate_simple(pix, 90);
+          g_object_unref( pix );
+          break;
+        default:
+          break;
+    }
+
+    DiscardData();
+#endif
+
+    return tmppixbuf;
+}
+
+static void updateTitle(const char *filename, MainWin *mw )
+{
+    static char fname[50];
+    static int wid, hei;
+    
+    char buf[100];
+    
+    if(filename != NULL)
+    {
+      strncpy(fname, filename, 49);
+      fname[49] = '\0';
+      
+      wid = gdk_pixbuf_get_width( mw->pix );
+      hei = gdk_pixbuf_get_height( mw->pix );
+    }
+    
+    snprintf(buf, 100, "%s (%dx%d) %d%%", fname, wid, hei, (int)(mw->scale * 100));
+    gtk_window_set_title( (GtkWindow*)mw, buf );
+    
+    return;    
+}
+
 gboolean main_win_open( MainWin* mw, const char* file_path, ZoomMode zoom )
 {
     GError* err = NULL;
+    GdkPixbufFormat* info;
+    info = gdk_pixbuf_get_file_info( file_path, NULL, NULL );
+    char* type = gdk_pixbuf_format_get_name( info );
 
     main_win_close( mw );
     mw->pix = gdk_pixbuf_new_from_file( file_path, &err );
+
     if( ! mw->pix )
     {
         main_win_show_error( mw, err->message );
         return FALSE;
     }
-
+    else if(!strcmp(type,"jpeg"))
+    {
+        // Only jpeg should rotate by EXIF
+        mw->pix = RotateByEXIF( file_path, mw->pix);
+    }
+    
     mw->zoom_mode = zoom;
 
     // select most suitable viewing mode
@@ -304,6 +410,7 @@ gboolean main_win_open( MainWin* mw, const char* file_path, ZoomMode zoom )
             gtk_widget_set_size_request( (GtkWidget*)mw->img_view, -1, -1 );
             gtk_scrolled_window_set_policy( (GtkScrolledWindow*)mw->scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
             mw->zoom_mode = ZOOM_ORIG;
+            mw->scale = 1.0;
         }
         else
             mw->zoom_mode = ZOOM_FIT;
@@ -323,7 +430,7 @@ gboolean main_win_open( MainWin* mw, const char* file_path, ZoomMode zoom )
     else  if( mw->zoom_mode == ZOOM_ORIG )  // original size
     {
         gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_orig, TRUE );
-        image_view_set_scale( (ImageView*)mw->img_view, 1.0, GDK_INTERP_BILINEAR );
+        image_view_set_scale( (ImageView*)mw->img_view, mw->scale, GDK_INTERP_BILINEAR );
         main_win_center_image( mw );
     }
 
@@ -335,7 +442,8 @@ gboolean main_win_open( MainWin* mw, const char* file_path, ZoomMode zoom )
     // build file list
     char* dir_path = g_path_get_dirname( file_path );
     image_list_open_dir( mw->img_list, dir_path, NULL );
-    image_list_sort_by_name( mw->img_list, GTK_SORT_ASCENDING );
+    //image_list_sort_by_name( mw->img_list, GTK_SORT_ASCENDING );
+    image_list_sort_by_name( mw->img_list, GTK_SORT_DESCENDING );
     g_free( dir_path );
 
     char* base_name = g_path_get_basename( file_path );
@@ -344,7 +452,7 @@ gboolean main_win_open( MainWin* mw, const char* file_path, ZoomMode zoom )
     char* disp_name = g_filename_display_name( base_name );
     g_free( base_name );
 
-    gtk_window_set_title( (GtkWindow*)mw, disp_name );
+    updateTitle( disp_name, mw );
     g_free( disp_name );
 
     return TRUE;
@@ -387,6 +495,27 @@ void on_size_allocate( GtkWidget* widget, GtkAllocation    *allocation )
     }
 }
 
+gboolean on_win_state_event( GtkWidget* widget, GdkEventWindowState* state )
+{
+    MainWin* mw = (MainWin*)widget;
+    if( state->new_window_state == GDK_WINDOW_STATE_FULLSCREEN )
+    {
+        static GdkColor black = {0};
+        gtk_widget_modify_bg( mw->evt_box, GTK_STATE_NORMAL, &black );
+        gtk_widget_hide( gtk_widget_get_parent(mw->nav_bar) );
+        mw->full_screen = TRUE;
+    }
+    else
+    {
+//        gtk_widget_reset_rc_styles( mw->evt_box );
+        static GdkColor white = {0, 65535, 65535, 65535};
+        gtk_widget_modify_bg( mw->evt_box, GTK_STATE_NORMAL, &white );
+        gtk_widget_show( gtk_widget_get_parent(mw->nav_bar) );
+        mw->full_screen = FALSE;
+    }
+    return TRUE;
+}
+
 void main_win_fit_size( MainWin* mw, int width, int height, gboolean can_strech, GdkInterpType type )
 {
     if( ! mw->pix )
@@ -407,6 +536,8 @@ void main_win_fit_size( MainWin* mw, int width, int height, gboolean can_strech,
     {
         mw->scale = 1.0;
         image_view_set_scale( (ImageView*)mw->img_view, 1.0, type );
+        
+        updateTitle(NULL, mw);
     }
 }
 
@@ -463,21 +594,9 @@ void on_zoom_fit( GtkToggleButton* btn, MainWin* mw )
 void on_full_screen( GtkWidget* btn, MainWin* mw )
 {
     if( ! mw->full_screen )
-    {
-        static GdkColor black = {0};
-        gtk_widget_modify_bg( mw->evt_box, GTK_STATE_NORMAL, &black );
-        gtk_widget_hide( gtk_widget_get_parent(mw->nav_bar) );
         gtk_window_fullscreen( (GtkWindow*)mw );
-    }
     else
-    {
-//        gtk_widget_reset_rc_styles( mw->evt_box );
-        static GdkColor white = {0, 65535, 65535, 65535};
-        gtk_widget_modify_bg( mw->evt_box, GTK_STATE_NORMAL, &white );
-        gtk_widget_show( gtk_widget_get_parent(mw->nav_bar) );
         gtk_window_unfullscreen( (GtkWindow*)mw );
-    }
-    mw->full_screen = ! mw->full_screen;
 }
 
 void on_orig_size_menu( GtkToggleButton* btn, MainWin* mw )
@@ -505,6 +624,9 @@ void on_orig_size( GtkToggleButton* btn, MainWin* mw )
 //    gtk_scrolled_window_set_policy( (GtkScrolledWindow*)mw->scroll,
 //                                     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
 
+    // update scale
+    updateTitle(NULL, mw);
+    
     gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_fit, FALSE );
 
     if( ! mw->pix )
@@ -518,7 +640,7 @@ void on_orig_size( GtkToggleButton* btn, MainWin* mw )
     main_win_center_image( mw ); // FIXME:  mw doesn't work well. Why?
 }
 
-void on_next( GtkWidget* btn, MainWin* mw )
+void on_prev( GtkWidget* btn, MainWin* mw )
 {
     const char* name;
     if( image_list_is_empty( mw->img_list ) )
@@ -540,7 +662,7 @@ void on_next( GtkWidget* btn, MainWin* mw )
     }
 }
 
-void on_prev( GtkWidget* btn, MainWin* mw )
+void on_next( GtkWidget* btn, MainWin* mw )
 {
     if( image_list_is_empty( mw->img_list ) )
         return;
@@ -561,10 +683,36 @@ void on_prev( GtkWidget* btn, MainWin* mw )
     }
 }
 
+//////////////////// rotate & flip
+
+static int trans_angle_to_id(int i)
+{
+    if(i == 0) 		return 1;
+    else if(i == 90)	return 6;
+    else if(i == 180)	return 3;
+    else if(i == 270)	return 8;
+    else if(i == -45)	return 7;
+    else if(i == -90)	return 2;
+    else if(i == -135)	return 5;
+    else if(i == -180)	return 4;
+}
+
+static int get_new_angle( int orig_angle, int rotate_angle )
+{
+    // defined in exif.c
+    extern int ExifRotateFlipMapping[9][9];
+    static int angle_trans_back[] = {0, 0, -90, 180, -180, -135, 90, -45, 270};
+    
+    orig_angle = trans_angle_to_id(orig_angle);
+    rotate_angle = trans_angle_to_id(rotate_angle);
+  
+    return angle_trans_back[ ExifRotateFlipMapping[orig_angle][rotate_angle] ];
+}
+
 void on_rotate_clockwise( GtkWidget* btn, MainWin* mw )
 {
     rotate_image( mw, GDK_PIXBUF_ROTATE_CLOCKWISE );
-    mw->rotation_angle += 90;
+    mw->rotation_angle = get_new_angle(mw->rotation_angle, 90);
     if(pref.auto_save_rotated){
         pref.ask_before_save = FALSE;
         on_save(btn,mw);
@@ -575,7 +723,7 @@ void on_rotate_clockwise( GtkWidget* btn, MainWin* mw )
 void on_rotate_counterclockwise( GtkWidget* btn, MainWin* mw )
 {
     rotate_image( mw, GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE );
-    mw->rotation_angle += 270;
+    mw->rotation_angle = get_new_angle(mw->rotation_angle, 270);
     if(pref.auto_save_rotated){
         pref.ask_before_save = FALSE;
         on_save(btn,mw);
@@ -583,6 +731,30 @@ void on_rotate_counterclockwise( GtkWidget* btn, MainWin* mw )
     }
 }
 
+void on_flip_vertical( GtkWidget* btn, MainWin* mw )
+{
+    rotate_image( mw, -180 );
+    mw->rotation_angle = get_new_angle(mw->rotation_angle, -180);
+    if(pref.auto_save_rotated){
+        pref.ask_before_save = FALSE;
+        on_save(btn,mw);
+        pref.ask_before_save = TRUE;
+    }
+}
+
+void on_flip_horizontal( GtkWidget* btn, MainWin* mw )
+{
+    rotate_image( mw, -90 );
+    mw->rotation_angle = get_new_angle(mw->rotation_angle, -90);
+    if(pref.auto_save_rotated){
+        pref.ask_before_save = FALSE;
+        on_save(btn,mw);
+        pref.ask_before_save = TRUE;
+    }
+}
+
+///////////////////// end of rotate & flip
+
 static void on_update_preview( GtkFileChooser *chooser, GtkImage* img )
 {
     char* file = gtk_file_chooser_get_preview_filename( chooser );
@@ -675,6 +847,16 @@ void on_save_as( GtkWidget* btn, MainWin* mw )
 
 #ifdef HAVE_LIBJPEG
 int rotate_and_save_jpeg_lossless(char *  filename,int angle){
+
+    char tmpfilename[] = {"/tmp/rot.jpg.XXXXXX"};
+    int tmpfilefd;
+    FILE *cp_in;
+    FILE *cp_out;
+    char cp_buf[512];
+
+    if(angle < 0)
+        return -1;
+    
     JXFORM_CODE code = JXFORM_NONE;
 
     angle = angle % 360;
@@ -686,17 +868,34 @@ int rotate_and_save_jpeg_lossless(char *  filename,int angle){
     else if(angle == 270)
         code = JXFORM_ROT_270;
 
+    /* create tmp file */
+    tmpfilefd = mkstemp(tmpfilename);
+    if (tmpfilefd == -1) {
+      return -1;
+    }
+    close(tmpfilefd);
+
     //rotate the image and save it to /tmp/rot.jpg
-    int error = jpegtran (filename, "/tmp/rot.jpg" , code);
+    int error = jpegtran (filename, tmpfilename , code);
     if(error)
         return error;
 
     //now copy /tmp/rot.jpg back to the original file
-    char command[strlen(filename)+50]; //this should not generate buffer owerflow
-    // MS: didn't know, how to make it better, maybe an own copy routine
-    sprintf(command,"cp /tmp/rot.jpg \"%s\"",filename);
-    system(command);
+    cp_in = fopen(tmpfilename,"rb");
+    cp_out = fopen(filename,"wb");
+    while (!feof(cp_in)) {
+      int n;
+      n = fread(cp_buf,1,sizeof(cp_buf),cp_in);
+      if (n<=0) {
+        break;
+      }
+      fwrite (cp_buf,1,n,cp_out);
+    }
+    fclose(cp_in);
+    fclose(cp_out);
 
+    /* remove tmp file */
+    unlink(tmpfilename);
     return 0;
 }
 #endif
@@ -711,12 +910,28 @@ void on_save( GtkWidget* btn, MainWin* mw )
     GdkPixbufFormat* info;
     info = gdk_pixbuf_get_file_info( file_name, NULL, NULL );
     char* type = gdk_pixbuf_format_get_name( info );
+
+    if(strcmp(type,"jpeg")==0)
+    {
+        if(!pref.rotate_exif_only || ExifRotate(file_name, mw->rotation_angle) == FALSE)
+        {
+            // hialan notes:
+            // ExifRotate retrun FALSE when 
+            //   1. Can not read file
+            //   2. Exif do not have TAG_ORIENTATION tag
+            //   3. Format unknown
+            // And then we apply rotate_and_save_jpeg_lossless() , 
+            // the result would not effected by EXIF Orientation...
 #ifdef HAVE_LIBJPEG
-    if(strcmp(type,"jpeg")==0){
-        if(rotate_and_save_jpeg_lossless(file_name,mw->rotation_angle)!=0)
-            main_win_show_error(mw, "Save failed! Check permissions.");
-    } else
+            if(rotate_and_save_jpeg_lossless(file_name,mw->rotation_angle)!=0)
+            {
+                main_win_show_error(mw, "Save failed! Check permissions.");
+            }
+#else
+            main_win_save( mw, file_name, type, pref.ask_before_save );
 #endif
+        }
+    } else
         main_win_save( mw, file_name, type, pref.ask_before_save );
     mw->rotation_angle = 0;
     g_free( file_name );
@@ -1025,14 +1240,26 @@ void main_win_center_image( MainWin* mw )
         gtk_adjustment_set_value(vadj, ( vadj->upper - vadj->page_size ) / 2 );
 }
 
-void rotate_image( MainWin* mw, GdkPixbufRotation angle )
+void rotate_image( MainWin* mw, int angle )
 {
+    GdkPixbuf* rpix = NULL;
+
     if( ! mw->pix )
         return;
 
-    GdkPixbuf* rpix = NULL;
-    rpix = gdk_pixbuf_rotate_simple( mw->pix, angle );
+    if(angle > 0)
+    {
+        rpix = gdk_pixbuf_rotate_simple( mw->pix, angle );
+    }
+    else
+    {
+        if(angle == -90)
+            rpix = gdk_pixbuf_flip( mw->pix, TRUE );
+        else if(angle == -180)
+            rpix = gdk_pixbuf_flip( mw->pix, FALSE );
+    }
     g_object_unref( mw->pix );
+    
     mw->pix = rpix;
     image_view_set_pixbuf( (ImageView*)mw->img_view, mw->pix );
 
@@ -1045,16 +1272,22 @@ gboolean main_win_scale_image( MainWin* mw, double new_scale, GdkInterpType type
     if( G_UNLIKELY( new_scale == 1.0 ) )
     {
         gtk_toggle_button_set_active( (GtkToggleButton*)mw->btn_orig, TRUE );
+        mw->scale = 1.0;
         return TRUE;
     }
     mw->scale = new_scale;
     image_view_set_scale( (ImageView*)mw->img_view, new_scale, type );
-
+    
+    updateTitle( NULL, mw );
+    
     return TRUE;
 }
 
 gboolean main_win_save( MainWin* mw, const char* file_path, const char* type, gboolean confirm )
 {
+    gboolean result1,gdk_save_supported;
+    GSList *gdk_formats;
+    GSList *gdk_formats_i;
     if( ! mw->pix )
         return FALSE;
 
@@ -1076,13 +1309,38 @@ gboolean main_win_save( MainWin* mw, const char* file_path, const char* type, gb
         }
     }
 
+    /* detect if the current type can be save by gdk_pixbuf_save() */
+    gdk_save_supported = FALSE;
+    gdk_formats = gdk_pixbuf_get_formats();
+    for (gdk_formats_i = gdk_formats; gdk_formats_i;
+         gdk_formats_i = g_slist_next(gdk_formats_i))
+    {
+        GdkPixbufFormat *data;
+        data = gdk_formats_i->data;
+        if (gdk_pixbuf_format_is_writable(data))
+        {
+            if ( strcmp(type, gdk_pixbuf_format_get_name(data))==0)
+            {
+                gdk_save_supported = TRUE;
+                break;
+            }
+        }
+    }
+    g_slist_free (gdk_formats);
+
     GError* err = NULL;
-    if( ! gdk_pixbuf_save( mw->pix, file_path, type, &err, NULL ) )
+    if (!gdk_save_supported)
+    {
+        /* FIXME: we should show some error messages here when the type
+           is not supported to save */
+        return FALSE;
+    }
+    result1 = gdk_pixbuf_save( mw->pix, file_path, type, &err, NULL );
+    if( ! result1 )
     {
         main_win_show_error( mw, err->message );
         return FALSE;
     }
-
     return TRUE;
 }
 
@@ -1125,10 +1383,13 @@ void show_popup_menu( MainWin* mw, GdkEventButton* evt )
         PTK_SEPARATOR_MENU_ITEM,
         PTK_IMG_MENU_ITEM( N_( "Rotate Counterclockwise" ), "gtk-counterclockwise", on_rotate_counterclockwise, GDK_L, 0 ),
         PTK_IMG_MENU_ITEM( N_( "Rotate Clockwise" ), "gtk-clockwise", on_rotate_clockwise, GDK_R, 0 ),
+//        PTK_IMG_MENU_ITEM( N_( "Flip Vertical" ), "gtk-clockwise", on_flip_vertical, GDK_V, 0 ),
+//        PTK_IMG_MENU_ITEM( N_( "Flip Horizontal" ), "gtk-clockwise", on_flip_horizontal, GDK_H, 0 ),
         PTK_SEPARATOR_MENU_ITEM,
         PTK_IMG_MENU_ITEM( N_("Open File"), GTK_STOCK_OPEN, G_CALLBACK(on_open), GDK_O, 0 ),
         PTK_IMG_MENU_ITEM( N_("Save File"), GTK_STOCK_SAVE, G_CALLBACK(on_save), GDK_S, 0 ),
         PTK_IMG_MENU_ITEM( N_("Save As"), GTK_STOCK_SAVE_AS, G_CALLBACK(on_save_as), GDK_A, 0 ),
+//        PTK_IMG_MENU_ITEM( N_("Save As Other Size"), GTK_STOCK_SAVE_AS, G_CALLBACK(on_save_as), GDK_A, 0 ),
         PTK_IMG_MENU_ITEM( N_("Delete File"), GTK_STOCK_DELETE, G_CALLBACK(on_delete), GDK_Delete, 0 ),
         PTK_SEPARATOR_MENU_ITEM,
         PTK_STOCK_MENU_ITEM( GTK_STOCK_ABOUT, on_about ),
@@ -1155,9 +1416,11 @@ static void open_url( GtkAboutDialog *dlg, const gchar *url, gpointer data)
         char* open_cmd = NULL;
         if( (open_cmd = g_find_program_in_path( programs[i] )) )
         {
-             char* cmd = g_strdup_printf( "%s \'%s\'", open_cmd, url );
-             g_spawn_command_line_async( cmd, NULL );
-             g_free( cmd );
+             gchar* argv [3];
+             argv [0] = programs[i];
+             argv [1] = url;
+             argv [2] = NULL;
+             g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL); 
              g_free( open_cmd );
              break;
         }
@@ -1171,6 +1434,7 @@ void on_about( GtkWidget* menu, MainWin* mw )
     {
         "洪任諭 Hong Jen Yee <pcman.tw@gmail.com>",
         "Martin Siggel <martinsiggel@googlemail.com>",
+        "Hialan Liu <hialan.liu@gmail.com>",
         _(" * Refer to source code of EOG image viewer and GThumb"),
         NULL
     };
@@ -1188,7 +1452,7 @@ void on_about( GtkWidget* menu, MainWin* mw )
     gtk_about_dialog_set_copyright ( (GtkAboutDialog*)about_dlg, _( "Copyright (C) 2007" ) );
     gtk_about_dialog_set_comments ( (GtkAboutDialog*)about_dlg, _( "Lightweight image viewer from LXDE project" ) );
     gtk_about_dialog_set_license ( (GtkAboutDialog*)about_dlg, "GPicView\n\nCopyright (C) 2007 Hong Jen Yee (PCMan)\n\nmw program is free software; you can redistribute it and/or\nmodify it under the terms of the GNU General Public License\nas published by the Free Software Foundation; either version 2\nof the License, or (at your option) any later version.\n\nmw program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License\nalong with mw program; if not, write to the Free Software\nFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA." );
-    gtk_about_dialog_set_website ( (GtkAboutDialog*)about_dlg, "http://lxde.sourceforge.net/gpicview/" );
+    gtk_about_dialog_set_website ( (GtkAboutDialog*)about_dlg, "http://lxde.org/gpicview/" );
     gtk_about_dialog_set_authors ( (GtkAboutDialog*)about_dlg, authors );
     gtk_about_dialog_set_translator_credits ( (GtkAboutDialog*)about_dlg, translators );
     gtk_window_set_transient_for( (GtkWindow*) about_dlg, GTK_WINDOW( mw ) );
